@@ -2,7 +2,7 @@
  * This file is part of gspell, a spell-checking library.
  *
  * Copyright 2002 - Paolo Maggi
- * Copyright 2015 - Sébastien Wilmet
+ * Copyright 2015, 2016 - Sébastien Wilmet
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -31,8 +31,9 @@
  * #GspellLanguage. #GspellLanguageChooserDialog implements the
  * #GspellLanguageChooser interface.
  *
- * The #GspellLanguageChooser:language property is updated only when the Select
- * button is pressed or when a row is activated (e.g. with a double-click).
+ * The #GspellLanguageChooser:language and #GspellLanguageChooser:language-code
+ * properties are updated only when the Select button is pressed or when a row
+ * is activated (e.g. with a double-click).
  *
  * The application is responsible to destroy the dialog, typically when the
  * #GtkDialog::response signal has been received or gtk_dialog_run() has
@@ -45,12 +46,14 @@ struct _GspellLanguageChooserDialogPrivate
 {
 	GtkTreeView *treeview;
 	const GspellLanguage *language;
+	guint default_language : 1;
 };
 
 enum
 {
 	PROP_0,
 	PROP_LANGUAGE,
+	PROP_LANGUAGE_CODE,
 };
 
 enum
@@ -94,7 +97,8 @@ scroll_to_selected (GtkTreeView *tree_view)
 }
 
 static const GspellLanguage *
-gspell_language_chooser_dialog_get_language (GspellLanguageChooser *chooser)
+gspell_language_chooser_dialog_get_language_full (GspellLanguageChooser *chooser,
+						  gboolean              *default_language)
 {
 	GspellLanguageChooserDialog *dialog;
 	GspellLanguageChooserDialogPrivate *priv;
@@ -102,15 +106,21 @@ gspell_language_chooser_dialog_get_language (GspellLanguageChooser *chooser)
 	dialog = GSPELL_LANGUAGE_CHOOSER_DIALOG (chooser);
 	priv = gspell_language_chooser_dialog_get_instance_private (dialog);
 
+	if (default_language != NULL)
+	{
+		*default_language = priv->default_language;
+	}
+
 	return priv->language;
 }
 
 static void
 gspell_language_chooser_dialog_set_language (GspellLanguageChooser *chooser,
-					     const GspellLanguage  *language)
+					     const GspellLanguage  *language_param)
 {
 	GspellLanguageChooserDialog *dialog;
 	GspellLanguageChooserDialogPrivate *priv;
+	const GspellLanguage *language;
 	GtkTreeSelection *selection;
 	GtkTreeModel *model;
 	GtkTreeIter iter;
@@ -118,18 +128,39 @@ gspell_language_chooser_dialog_set_language (GspellLanguageChooser *chooser,
 	dialog = GSPELL_LANGUAGE_CHOOSER_DIALOG (chooser);
 	priv = gspell_language_chooser_dialog_get_instance_private (dialog);
 
+	language = language_param;
+
+	if (language == NULL)
+	{
+		language = gspell_language_get_default ();
+	}
+
 	selection = gtk_tree_view_get_selection (priv->treeview);
 
 	if (language == NULL)
 	{
+		gboolean notify_language_code = FALSE;
+
 		gtk_tree_selection_unselect_all (selection);
 
+		/* Update first the full state before notifying the properties. */
+		if (!priv->default_language)
+		{
+			priv->default_language = TRUE;
+			notify_language_code = TRUE;
+		}
+
 		if (priv->language != NULL)
 		{
 			priv->language = NULL;
 			g_object_notify (G_OBJECT (dialog), "language");
 		}
 
+		if (notify_language_code)
+		{
+			g_object_notify (G_OBJECT (dialog), "language-code");
+		}
+
 		return;
 	}
 
@@ -150,13 +181,32 @@ gspell_language_chooser_dialog_set_language (GspellLanguageChooser *chooser,
 
 		if (language == cur_lang)
 		{
+			gboolean default_language;
+			gboolean notify_language_code = FALSE;
+
 			gtk_tree_selection_select_iter (selection, &iter);
 			scroll_to_selected (priv->treeview);
 
+			/* Update first the full state before notifying the properties. */
+
+			default_language = language_param == NULL;
+
+			if (priv->default_language != default_language)
+			{
+				priv->default_language = default_language;
+				notify_language_code = TRUE;
+			}
+
 			if (priv->language != language)
 			{
 				priv->language = language;
 				g_object_notify (G_OBJECT (dialog), "language");
+				notify_language_code = TRUE;
+			}
+
+			if (notify_language_code)
+			{
+				g_object_notify (G_OBJECT (dialog), "language-code");
 			}
 
 			return;
@@ -171,7 +221,7 @@ warning:
 static void
 gspell_language_chooser_dialog_iface_init (GspellLanguageChooserInterface *iface)
 {
-	iface->get_language = gspell_language_chooser_dialog_get_language;
+	iface->get_language_full = gspell_language_chooser_dialog_get_language_full;
 	iface->set_language = gspell_language_chooser_dialog_set_language;
 }
 
@@ -186,7 +236,11 @@ gspell_language_chooser_dialog_get_property (GObject    *object,
 	switch (prop_id)
 	{
 		case PROP_LANGUAGE:
-			g_value_set_boxed (value, gspell_language_chooser_dialog_get_language (chooser));
+			g_value_set_boxed (value, gspell_language_chooser_get_language (chooser));
+			break;
+
+		case PROP_LANGUAGE_CODE:
+			g_value_set_string (value, gspell_language_chooser_get_language_code (chooser));
 			break;
 
 		default:
@@ -206,7 +260,11 @@ gspell_language_chooser_dialog_set_property (GObject      *object,
 	switch (prop_id)
 	{
 		case PROP_LANGUAGE:
-			gspell_language_chooser_dialog_set_language (chooser, g_value_get_boxed (value));
+			gspell_language_chooser_set_language (chooser, g_value_get_boxed (value));
+			break;
+
+		case PROP_LANGUAGE_CODE:
+			gspell_language_chooser_set_language_code (chooser, g_value_get_string (value));
 			break;
 
 		default:
@@ -246,6 +304,7 @@ dialog_response_cb (GtkDialog *gtk_dialog,
 	GtkTreeModel *model;
 	GtkTreeIter iter;
 	const GspellLanguage *lang;
+	gboolean notify_language_code = FALSE;
 
 	if (response != GTK_RESPONSE_OK)
 	{
@@ -266,10 +325,24 @@ dialog_response_cb (GtkDialog *gtk_dialog,
 			    COLUMN_LANGUAGE_POINTER, &lang,
 			    -1);
 
+	/* Update first the full state before notifying the properties. */
+
+	if (priv->default_language)
+	{
+		priv->default_language = FALSE;
+		notify_language_code = TRUE;
+	}
+
 	if (priv->language != lang)
 	{
 		priv->language = lang;
 		g_object_notify (G_OBJECT (dialog), "language");
+		notify_language_code = TRUE;
+	}
+
+	if (notify_language_code)
+	{
+		g_object_notify (G_OBJECT (dialog), "language-code");
 	}
 }
 
@@ -284,6 +357,7 @@ gspell_language_chooser_dialog_class_init (GspellLanguageChooserDialogClass *kla
 	object_class->constructed = gspell_language_chooser_dialog_constructed;
 
 	g_object_class_override_property (object_class, PROP_LANGUAGE, "language");
+	g_object_class_override_property (object_class, PROP_LANGUAGE_CODE, "language-code");
 
 	/* Bind class to template */
 	gtk_widget_class_set_template_from_resource (widget_class, "/org/gnome/gspell/language-dialog.ui");
@@ -334,17 +408,23 @@ gspell_language_chooser_dialog_init (GspellLanguageChooserDialog *dialog)
 {
 	GspellLanguageChooserDialogPrivate *priv;
 	GtkListStore *store;
+	GtkTreeSelection *selection;
 	GtkTreeViewColumn *column;
 	GtkCellRenderer *renderer;
 
 	priv = gspell_language_chooser_dialog_get_instance_private (dialog);
 
+	priv->default_language = TRUE;
+
 	gtk_widget_init_template (GTK_WIDGET (dialog));
 
 	store = gtk_list_store_new (N_COLUMNS, G_TYPE_STRING, GSPELL_TYPE_LANGUAGE);
 	gtk_tree_view_set_model (priv->treeview, GTK_TREE_MODEL (store));
 	g_object_unref (store);
 
+	selection = gtk_tree_view_get_selection (priv->treeview);
+	gtk_tree_selection_set_mode (selection, GTK_SELECTION_BROWSE);
+
 	/* Add the language column */
 	column = gtk_tree_view_column_new ();
 	renderer = gtk_cell_renderer_text_new ();
@@ -389,7 +469,8 @@ gspell_language_chooser_dialog_init (GspellLanguageChooserDialog *dialog)
 /**
  * gspell_language_chooser_dialog_new:
  * @parent: transient parent of the dialog.
- * @current_language: the #GspellLanguage to select initially.
+ * @current_language: (nullable): the #GspellLanguage to select initially, or
+ *   %NULL to pick the default language.
  * @flags: #GtkDialogFlags
  *
  * Returns: a new #GspellLanguageChooserDialog widget.
